#include "mellanox_i2c_access.h"

#include <stdio.h>
#include <stdlib.h>
#include "mlnx_if_us.h"

static const unsigned int mellanox_i2c_bus = 1;

int eeprom_open(char *dev_fqn __attribute__((unused)), int addr, int type __attribute__((unused)), struct eeprom* e __attribute__((unused)))
{
    char mux_address_pin = 0x40;
    char data;
    int rc;

    rc = system("/etc/init.d/sx_low_level_if start 1>/dev/null");
    if (rc < 0) {
        fprintf(stderr, "Could not /etc/init.d/sx_low_level_if start\n");
        return rc;
    }

    if (addr != 0x51) {
        fprintf(stderr, "Only address 0x51 is supported for Mellanox eeprom\n");
        return -1;
    }
    rc = mlnx_i2c_read(mellanox_i2c_bus, 0x70, 0, 0, &data, 1);
    if (rc < 0) {
        /* cpld mux flow */
        rc = system("iorw -b 9691 -w -l1 -v 8 1>/dev/null");
        if (rc < 0) {
            fprintf(stderr, "Could not iorw cpld mux to open 0x51 mux path\n");
            return rc;
        }
        rc = system("iorw -b 9691 -r -l1 1>/dev/null");
        if (rc < 0) {
            fprintf(stderr, "Could not verify iorw read\n");
            return rc;
        }
    } else {
        /* non-cpld mux flow */
        rc = mlnx_i2c_write(mellanox_i2c_bus, 0x70, 0, 0, &mux_address_pin, 1);
        if (rc < 0) {
            fprintf(stderr, "Could not set Mux path to 0x51\n");
            return rc;
        }
    }

    rc = mlnx_i2c_read(mellanox_i2c_bus, 0x51, 1, 2, &data, 1);
    if (rc < 0) {
        fprintf(stderr, "Could read from 0x51 address after eeprom opening\n");
    }
    return rc;
#ifdef DEBUG
    printf("%s : dev_fqn %p addr %d type %d eeprom %p\n" ,__func__, dev_fqn , addr , type , e);
#endif
    return 0;
}
int eeprom_close(struct eeprom *e __attribute__((unused)))
{
    int rc;
    rc = system("/etc/init.d/sx_low_level_if stop 1>/dev/null");
    if (rc < 0) {
        fprintf(stderr, "Warning: could not /etc/init.d/sx_low_level_if stop\n");
    }
#ifdef DEBUG
    printf("%s : eeprom %p\n" ,__func__,e);
#endif
    return 0;
}
int eeprom_read_byte(struct eeprom* e __attribute__((unused)), unsigned int mem_addr)
{
    char data;
    int rc;

    rc = mlnx_i2c_read(mellanox_i2c_bus, 0x51, mem_addr, 2, &data, 1);
        if (rc < 0) {
            fprintf(stderr, "Read from address 0x51 offset %d failed\n", mem_addr);
            return 0;
        }
#ifdef DEBUG
    printf("%s : eeprom %p mem_addr %d\n" ,__func__, e , mem_addr);
    printf("data read: %hhx \n", data);
    printf ("rc = %d\n" , rc);
#endif
    return data;
}
int eeprom_write_byte(struct eeprom *e __attribute__((unused)), unsigned int mem_addr, unsigned int data)
{
    int rc;
    char output_data;
    output_data = data;
    rc = mlnx_i2c_write(mellanox_i2c_bus, 0x51, mem_addr, 2, &output_data, 1);
#ifdef DEBUG
    printf("%s : eeprom %p mem_addr %d data %d\n" ,__func__, e, mem_addr, data);
    printf("data written: %hhx \n", data);
    printf ("rc = %d\n" , rc);
#endif
    return rc;
}

